﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Backend.Csv {
  [StorableClass]
  public class CsvImportCommand : DataSetCommandBase {
    private ColumnGroup newColumnGroup;

    private CsvImportCommand()
      : base(null) {
    }

    public CsvImportCommand(DataSet dataSet)
      : base(dataSet) {
    }

    [Storable]
    private string fileName;
    public string FileName {
      get { return fileName; }
      set { this.fileName = value; }
    }

    [Storable]
    private char[] delimiter;
    public char[] Delimiter {
      get { return this.delimiter; }
      set { this.delimiter = value; }
    }

    [Storable]
    private bool columnNamesInFirstRow;
    public bool ColumnNamesInFirstRow {
      get { return this.columnNamesInFirstRow; }
      set { this.columnNamesInFirstRow = value; }
    }

    public override string Description {
      get { return "Import CSV file"; }
    }

    public override void Execute() {
      base.Execute();
      newColumnGroup = new ColumnGroup(fileName.Substring(fileName.LastIndexOf('\\') + 1));
      StreamReader reader = new StreamReader(fileName);
      string firstLine = reader.ReadLine();

      int i = 1;
      string[] values = firstLine.Split(delimiter);
      foreach (string value in values) {
        if (columnNamesInFirstRow)
          newColumnGroup.AddColumn(new StringColumn(value));
        else {
          newColumnGroup.AddColumn(new StringColumn("Column " + i));
        }
        i++;
      }
      if (!columnNamesInFirstRow)
        newColumnGroup.AddRow(values);
      while (!reader.EndOfStream) {
        newColumnGroup.AddRow(reader.ReadLine().Split(delimiter));
      }
      reader.Close();
      DataSet.AddColumnGroup(newColumnGroup);
      DataSet.FireChanged();
    }

    public override void UndoExecute() {
      base.UndoExecute();
      DataSet.RemoveColumnGroup(newColumnGroup);
      newColumnGroup = null;
      DataSet.FireChanged();
    }
  }
}
