﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Delete Rows", 1, ColumnGroupState.AnySelectedColumnContainsNull,
    "Handle Missing Values", Position = 4)]
  public class DeleteRowsWithMissingValuesCommand : ColumnGroupCommandWithAffectedColumnsBase {
    private List<ColumnBase> oldColumns;
    private ICollection<int> oldSortedColumnIndices;

    private DeleteRowsWithMissingValuesCommand()
      : base(null, string.Empty, null) {
      oldColumns = new List<ColumnBase>();
    }

    public DeleteRowsWithMissingValuesCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
      oldColumns = new List<ColumnBase>();
    }

    public override void Execute() {
      base.Execute();
      ColumnGroup tempColumnGroup = new ColumnGroup();
      Dictionary<int, object> positions = new Dictionary<int, object>();
      ColumnBase column;

      for (int col = 0; col < AffectedColumns.Length; col++) {
        column = ColumnGroup.GetColumn(AffectedColumns[col]);
        for (int row = 0; row < ColumnGroup.RowCount; row++) {
          if (column.GetValue(row) == null && !positions.ContainsKey(row))
            positions[row] = null;
        }
      }

      foreach (ColumnBase x in ColumnGroup.Columns) {
        oldColumns.Add(x);
        tempColumnGroup.AddColumn(x.CreateCopyOfColumnWithoutValues());
      }

      for (int i = 0; i < ColumnGroup.RowCount; i++)
        if (!positions.ContainsKey(i))
          tempColumnGroup.AddRow(ColumnGroup.GetRow(i));

      oldSortedColumnIndices = new List<int>(ColumnGroup.SortedColumnIndexes);
      for (int i = 0; i < ColumnGroup.Columns.Count(); i++) {
        tempColumnGroup.Columns.ElementAt(i).SortOrder = ColumnGroup.GetColumn(i).SortOrder;
        ColumnGroup.ReplaceColumn(i, tempColumnGroup.Columns.ElementAt(i));
      }

      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndices;
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      for (int i = 0; i < ColumnGroup.Columns.Count(); i++)
        ColumnGroup.ReplaceColumn(i, oldColumns[i]);
      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndices;
      oldColumns.Clear();
      oldSortedColumnIndices = null;
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    public override string Description {
      get { return "Delete rows with missing values"; }
    }
  }
}
