﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  public abstract class FillMissingValueCommandBase : ColumnGroupCommandWithAffectedColumnsBase {
    protected Dictionary<int, List<int>> nullValues;
    protected ICollection<int> oldSortedColumnIndexes;
    protected IEnumerable<SortOrder> oldSortOrder;

    private FillMissingValueCommandBase()
      : base(null, string.Empty, null) {
      nullValues = new Dictionary<int, List<int>>();
    }

    protected FillMissingValueCommandBase(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
      nullValues = new Dictionary<int, List<int>>();
    }

    public override void UndoExecute() {
      base.UndoExecute();
      ColumnBase column;
      foreach (int col in nullValues.Keys) {
        column = ColumnGroup.GetColumn(col);
        for (int j = 0; j < this.nullValues[col].Count; j++)
          column.ChangeValue(this.nullValues[col][j], null);
      }
      ColumnGroup.SortOrdersForColumns = oldSortOrder;
      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndexes;
      oldSortedColumnIndexes = null;
      oldSortOrder = null;
      nullValues.Clear();
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }
  }
}
