﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Linear Interpolation", 1, ColumnGroupState.DoubleColumnSelected | ColumnGroupState.AnySelectedColumnContainsNull,
    "Handle Missing Values", Position = 2)]
  public class FillMissingValuesWithLinearInterpolation : FillMissingValueCommandBase {
    private FillMissingValuesWithLinearInterpolation()
      : base(null, string.Empty, null) {
    }

    public FillMissingValuesWithLinearInterpolation(DataSet dataSet, string columnGroupName, int[] affectedColumns) :
      base(dataSet, columnGroupName, affectedColumns) {
    }

    public override string Description {
      get { return "Interpolate missing values"; }
    }

    public override void Execute() {
      base.Execute();
      DoubleColumn column;
      int startIndex = -1;
      int endIndex = -1;
      oldSortOrder = ColumnGroup.SortOrdersForColumns.ToList();
      oldSortedColumnIndexes = new List<int>(ColumnGroup.SortedColumnIndexes);
      foreach (int col in AffectedColumns) {
        if (ColumnGroup.GetColumn(col) is DoubleColumn && ColumnGroup.GetColumn(col).ContainsNullValues) {
          column = (DoubleColumn)ColumnGroup.Columns.ElementAt(col);
          if (column.SortOrder != SortOrder.None)
            ColumnGroup.ResetSorting();
          nullValues.Add(col, new List<int>());
          startIndex = FindNextNotNullValueIndex(column, startIndex);
          endIndex = FindNextNotNullValueIndex(column, startIndex);
          while (endIndex != -1) {
            if (endIndex - startIndex != 1) {
              nullValues[col].AddRange(Enumerable.Range(startIndex + 1, endIndex - startIndex - 1));
              InterpolateValues(column, startIndex, endIndex);
            }
            startIndex = endIndex;
            endIndex = FindNextNotNullValueIndex(column, endIndex);
          }
          startIndex = -1;
        }
      }
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    private int FindNextNotNullValueIndex(DoubleColumn column, int startIndex) {
      int endIndex = -1;
      for (int i = startIndex + 1; i < column.TotalValuesCount && endIndex == -1; i++)
        if (column.GetValue(i) != null)
          endIndex = i;
      return endIndex;
    }

    private void InterpolateValues(DoubleColumn column, int startIndex, int endIndex) {
      double startValue = (double)column.GetValue(startIndex);
      double endValue = (double)column.GetValue(endIndex);
      double stepwidth = (endValue - startValue) / (endIndex - startIndex);

      for (int i = startIndex + 1; i < endIndex; i++) {
        if (column.GetValue(i) != null)
          throw new CommandExecutionException("Tried to manipulate non null value!", this);
        column.ChangeValue(i, startValue + stepwidth * (i - startIndex));
      }
    }
  }
}
