﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Median", 1, ColumnGroupState.DoubleColumnSelected | ColumnGroupState.AnySelectedColumnContainsNull,
    "Handle Missing Values", Position = 1)]
  public class FillMissingValuesWithMedianCommand : FillMissingValueCommandBase {
    private FillMissingValuesWithMedianCommand()
      : base(null, string.Empty, null) {
    }

    public FillMissingValuesWithMedianCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns) :
      base(dataSet, columnGroupName, affectedColumns) {
    }

    public override string Description {
      get { return "Fill missing values with median"; }
    }

    public override void Execute() {
      base.Execute();
      DoubleColumn column;
      double? median;
      oldSortOrder = ColumnGroup.SortOrdersForColumns.ToList();
      oldSortedColumnIndexes = new List<int>(ColumnGroup.SortedColumnIndexes);
      foreach (int col in AffectedColumns) {
        if (ColumnGroup.GetColumn(col) is DoubleColumn && ColumnGroup.GetColumn(col).ContainsNullValues) {
          column = (DoubleColumn)ColumnGroup.Columns.ElementAt(col);
          if (column.SortOrder != SortOrder.None)
            ColumnGroup.ResetSorting();
          nullValues.Add(col, new List<int>());
          median = column.Median;
          for (int j = 0; j < column.TotalValuesCount; j++) {
            if (column.GetValue(j) == null) {
              column.ChangeValue(j, median);
              this.nullValues[col].Add(j);
            }
          }
        }
      }
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }
  }
}
