﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Previous Value", 1, ColumnGroupState.AnySelectedColumnContainsNull, "Handle Missing Values", Position = 3)]
  public class FillMissingValuesWithPreviousValuesCommand : FillMissingValueCommandBase {
    private FillMissingValuesWithPreviousValuesCommand()
      : base(null, string.Empty, null) {
    }

    public FillMissingValuesWithPreviousValuesCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns) :
      base(dataSet, columnGroupName, affectedColumns) {
    }

    public override void Execute() {
      base.Execute();
      IComparable val = null;
      ColumnBase column;
      oldSortOrder = ColumnGroup.SortOrdersForColumns.ToList();
      oldSortedColumnIndexes = new List<int>(ColumnGroup.SortedColumnIndexes);
      foreach (int col in AffectedColumns) {
        column = ColumnGroup.GetColumn(col);
        if (column.ContainsNullValues) {
          if (column.SortOrder != SortOrder.None)
            ColumnGroup.ResetSorting();
          nullValues.Add(col, new List<int>());
          val = null;
          for (int j = 0; j < column.TotalValuesCount; j++) {
            if (column.GetValue(j) != null)
              val = column.GetValue(j);
            else {
              column.ChangeValue(j, val);
              this.nullValues[col].Add(j);
            }
          }
        }
      }
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    public override string Description {
      get { return "Fill missing values with previous value"; }
    }
  }
}
