﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HeuristicLab.DataImporter.Data.CommandBase {
  [AttributeUsage(AttributeTargets.Class,AllowMultiple=false,Inherited=false)]
  public sealed class ViewableCommandInfoAttribute : Attribute{
    public ViewableCommandInfoAttribute(string name, int activeColumnGroups, ColumnGroupState state, string groupName) {
      this.name = name;
      this.groupName = groupName;
      this.state = new DataSetState(activeColumnGroups, state);
      this.position = Int32.MaxValue;
    }

    private DataSetState state;
    public DataSetState State {
      get { return this.state; }
    }

    private string groupName;
    public string GroupName {
      get { return this.groupName; }
    }

    private string name;
    public string Name {
      get { return this.name; }
    }

    private int position;
    public int Position {
      get { return this.position; }
      set { this.position = value; }
    }

    private Type optionsView;
    public Type OptionsView {
      get { return this.optionsView; }
      set {
        if (!value.IsSubclassOf(typeof(CommandViewBase)) || value.IsAbstract)
          throw new ArgumentException("Expected type T with base class CommandInfoViewBase");
        this.optionsView = value; }
    }

    public int SelectedColumns {
      get { return this.state.SelectedColumns; }
      set { this.state.SelectedColumns = value; }
    }

    public int MinActiveColumnGroups {
      get { return state.MinActiveColumnGroups; }
      set { state.MinActiveColumnGroups = value; }
    }
  }
}
