﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using HeuristicLab.DataImporter.Data.View;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Data.Model {
  [StorableClass]
  public class DataSet {

    [Storable]
    private List<ColumnGroup> columnGroups;
    public IEnumerable<ColumnGroup> ColumnGroups {
      get { return this.columnGroups; }
    }

    public DataSet() {
      this.columnGroups = new List<ColumnGroup>();
    }

    public event EventHandler Changed;
    protected virtual void OnChanged() {
      if (this.Changed != null)
        this.Changed(this, EventArgs.Empty);
    }
    public void FireChanged() {
      this.OnChanged();
    }

    public void AddColumnGroup(ColumnGroup columnGroup) {
      this.CheckColumnGroupName(columnGroup);
      this.columnGroups.Add(columnGroup);
    }

    public void AddColumnGroups(IEnumerable<ColumnGroup> columnGroups) {
      foreach (ColumnGroup columnGroup in columnGroups) {
        this.CheckColumnGroupName(columnGroup);
      }
      this.columnGroups.AddRange(columnGroups);
    }

    private void CheckColumnGroupName(ColumnGroup columnGroup) {
      int i = 0;
      string columnGroupName = columnGroup.Name;
      while (this.ColumnGroups.Any(cg => cg.Name == columnGroupName && cg != columnGroup)) {
        i++;
        columnGroupName = columnGroup.Name + " " + i;
      }
      columnGroup.Name = columnGroupName;
    }

    public void InsertColumnGroup(int position, ColumnGroup columnGroup) {
      this.CheckColumnGroupName(columnGroup);
      this.columnGroups.Insert(position, columnGroup);
    }

    public void ReplaceColumnGroup(int position, ColumnGroup columnGroup) {
      this.columnGroups[position] = columnGroup;
      this.CheckColumnGroupName(columnGroup);
    }

    public void RemoveColumnGroup(ColumnGroup columnGroup) {
      this.columnGroups.Remove(columnGroup);
    }

    public int IndexOfColumnGroup(ColumnGroup columnGroup) {
      return this.columnGroups.IndexOf(columnGroup);
    }

    public ColumnGroup GetColumnGroup(string columnGroupName) {
      IEnumerable<ColumnGroup> groups = this.columnGroups.Where(g => g.Name == columnGroupName);
      if (groups.Count() != 1)
        throw new InvalidOperationException("Found 0 or more than 1 column group with the name \"" + columnGroupName + "\".");
      return groups.First();

    }

    public IEnumerable<ColumnGroup> ActiveColumnGroups {
      get {
        return this.columnGroups.Where(cg => cg.Active);
      }
    }
  }
}
