﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Globalization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Data.Model {
  [StorableClass]
  public class DateTimeColumn : ColumnBase {
    [Storable]
    private List<DateTime?> values;

    private DateTimeColumn() : base(string.Empty) {
      this.DataType = typeof(DateTime?);
    }

    public DateTimeColumn(string columnName)
      : base(columnName) {
      this.DataType = typeof(DateTime?);
      this.values = new List<DateTime?>();
    }

    public DateTimeColumn(string columnName, int capacity)
      : this(columnName) {
      this.values.Capacity = capacity;
    }

    public override string ToString() {
      return base.ToString() + " <Date>";
    }

    protected override System.Collections.IList Values {
      get { return this.values; }
    }

    public override void AddValue(IComparable value) {
      this.values.Add((DateTime?)value);
    }

    public override void AddValueOrNull(IComparable value) {
      this.values.Add(ConvertToDateTime(value));
    }

    public override void ChangeValue(int position, IComparable value) {
      this.values[position] = (DateTime?)value;
    }

    public override void ChangeValueOrNull(int position, IComparable value) {
      this.values[position] = ConvertToDateTime(value);
    }

    public override void ChangeValueOrLeaveOldValue(int position, IComparable value) {
      if (value == null)
        this.values[position] = null;
      else {
        DateTime? val = ConvertToDateTime(value);
        if (val != null)
          this.values[position] = val;
      }
    }

    public override void InsertValue(int position, IComparable value) {
      this.values.Insert(position, (DateTime?)value);
    }

    private DateTime? ConvertToDateTime(IComparable value) {
      DateTime val;

      if (value == null)
        return null;

      if (DateTime.TryParse(value.ToString(), out val))
        return val;

      if (DateTime.TryParseExact(value.ToString(), "yyyyMMddHHmmss", CultureInfo.InvariantCulture, DateTimeStyles.None, out val))
        return val;

      return null;
    }

    public override ColumnBase CreateCopyOfColumnWithoutValues() {
      return CreateCopyOfColumnWithoutValues(this.values.Capacity);
    }

    public override ColumnBase CreateCopyOfColumnWithoutValues(int capacity) {
      return new DateTimeColumn(this.Name, capacity);
    }
  }
}
