﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;
using System.Xml;
using System.Globalization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Data.Model {
  [StorableClass]
  public class StringColumn : ColumnBase {
    [Storable]
    private List<string> values;

    private StringColumn() :base(string.Empty) {
      this.DataType = typeof(string);
    }

    public StringColumn(string columnName)
      : base(columnName) {
      this.DataType = typeof(string);
      this.values = new List<string>();
    }

    public StringColumn(string columnName, int capacity)
      : this(columnName) {
      this.values.Capacity = capacity;
    }

    public override string ToString() {
      return base.ToString() + " <String>";
    }

    protected override IList Values {
      get { return this.values; }
    }

    public override void AddValue(IComparable value) {
      this.values.Add(CheckInput(value));
    }

    public override void AddValueOrNull(IComparable value) {
      if (value == null)
        this.values.Add(null);
      else
        this.values.Add(CheckInput(value.ToString()));
    }

    public override void InsertValue(int position, IComparable value) {
      this.values.Insert(position, CheckInput(value));
    }

    public override void ChangeValue(int position, IComparable value) {
      this.values[position] = CheckInput(value);
    }

    public override void ChangeValueOrNull(int position, IComparable value) {
      this.values[position] = CheckInput(value);
    }

    public override void ChangeValueOrLeaveOldValue(int position, IComparable value) {
      this.values[position] = CheckInput(value);
    }

    public override ColumnBase CreateCopyOfColumnWithoutValues() {
      return CreateCopyOfColumnWithoutValues(this.values.Capacity);
    }
    public override ColumnBase CreateCopyOfColumnWithoutValues(int capacity) {
      return new StringColumn(this.Name, capacity);
    }

    private string CheckInput(IComparable value) {
      if (value == null)
        return null;
      string tmp = value.ToString().Trim();
      if (string.IsNullOrEmpty(tmp))
        tmp = null;
      return tmp;
    }
  }
}
